/*globals window, document, RSVP, rJS,
         location, console*/
/*jslint indent: 2, maxlen: 80*/
(function (window, document, RSVP, rJS, location, console, JSZip) {
  "use strict";

  function getExtension(url) {
    var extension = url.split('.').pop();
    if (extension.endsWith('/')) {
      return ".html";
    }
    return "." + extension;
  }

  function getSetting(gadget, key, default_value) {
    if ( key === "site_editor_gadget_url" ) {
      return window.location.protocol + "//" + window.location.host + "/crib-enable.html"
    } 
    return default_value;
  }

  function loadContentFromZIPURL(gadget, options) {
    var path_to_load = options.to_path, path_to_load_length, file_list, crib_sw_gadget,
      from_path = options.from_path, zip_url = options.zip_url,
      jio_gadget, url_list = [], url_number = 0;
    return new RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget('crib_sw_gadget');
    })
    .push(function (returned_gadget) {
      crib_sw_gadget = returned_gadget;
      return fetch(zip_url)
        .then(function (response) {                       // 2) filter on 200 OK
            if (response.status === 200 || response.status === 0) {
                return Promise.resolve(response.blob());
            } else {
                return Promise.reject(new Error(response.statusText));
            }
        });
    })
    .push(JSZip.loadAsync)
    .push(function (zip) {
      var promise_list = [];
      zip.forEach(function (relativePath, zipEntry) {
        var end_url;
        url_number += 1;
        if ( zipEntry.dir ) {
          return
        }
        if ( !relativePath.startsWith(from_path) ) {
          return
        }
        relativePath = relativePath.substring(from_path.length);
        if (!relativePath.startsWith("/") && !path_to_load.endsWith("/")) {
          end_url = path_to_load + "/" + relativePath 
        } else if (relativePath.startsWith("/") && path_to_load.endsWith("/")) {
          end_url = path_to_load + relativePath.substring(1);
        } else {
          end_url = path_to_load + relativePath 
        }
        promise_list.push(
          new RSVP.Queue().
            push(function () {
              return zipEntry.async('blob');
            })
            .push(function (result) {
              if ( end_url.endsWith(".js") ) {
                // This is a ugly hack as mimetype needs to be correct for JS
                result = result.slice(0, result.size, "application/javascript")
              } else if ( end_url.endsWith(".html") ) {
                // This is a ugly hack as mimetype needs to be correct for JS
                result = result.slice(0, result.size, "text/html")
              } else if ( end_url.endsWith(".css") ) {
                // This is a ugly hack as mimetype needs to be correct for JS
                result = result.slice(0, result.size, "text/css")
              } 
              return crib_sw_gadget.put(end_url, {blob: result})
            })
        )
      })
      return RSVP.all(promise_list)
    })
    .push(console.log, console.log)
  }

  rJS(window)
    .ready(function (g) {
    g.props = {};
  })
    .declareMethod('loadFromZipUrl', function (options) {
      return loadContentFromZIPURL(this, options);
    })
    .declareMethod('loadFromZipFile', function (options) {
      return loadContentFromZIPURL(gadget, options);
    })
    .allowPublicAcquisition("getSetting", function (argument_list) {
      return getSetting(this, argument_list[0], argument_list[1]);
    })

}(window, document, RSVP, rJS, location, console, JSZip));